//+------------------------------------------------------------------+
//|                                                     evolveNN.mq4 |
//|                                         Copyright  2009, m_OO_m |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, m_OO_m"
#property link      ""

//---- input parameters
extern int       HistoryBars=10000;
extern int       TrainingIntervalBars=1000;
extern double    lots=1;
extern int       CreateTrainingDataset=0;
int TrainingCounter;
int lastBars = 0;
int ticketLong = 0;
int ticketShort = 0;
double EinstiegLong = 0;
double EinstiegShort = 0;
double OpenAtRequest = 0;
bool training = false;
double spread = 0;
double stoplevel = 0;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   TrainingCounter = TrainingIntervalBars + 1;
   spread = MarketInfo(Symbol(),MODE_SPREAD);
   stoplevel = MarketInfo(Symbol(),MODE_STOPLEVEL);   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+

int start()
  {
//----
   int handle;
   int err;
      
   if(TrainingCounter > TrainingIntervalBars && ticketLong == 0 && ticketShort == 0 && !training) {
      training = true;//Lernmodus aktiv 
      
      if(CreateTrainingDataset == 1) {
         Print("Writing training data.");
      
         //Lerndatensatz generieren
         handle = FileOpen(Symbol() + Period() + ".csv", FILE_CSV|FILE_WRITE);
         FileWrite(handle, Point, spread, stoplevel);//1. Zeile: Point, Spread, StopLevel
      
         for(int i = HistoryBars - 1; i >= 0; i--) {
            writeLine(i, handle, true);
         }   
      
         FileClose(handle);
      } else {
         Print("Training dataset skipped");
      }
            
      return(0);
   }
      
   //Prfen, ob die Orders noch da sind
   if(ticketLong != 0 && OrderSelect(ticketLong, SELECT_BY_TICKET) && OrderCloseTime() != 0) {
      Print("Stop loss on ticket No. " + ticketLong + " (long)");
      ticketLong = 0;
      EinstiegLong = 0;
   }
   if(ticketShort != 0 && OrderSelect(ticketShort, SELECT_BY_TICKET) && OrderCloseTime() != 0) {
      Print("Stop loss on ticket No. " + ticketShort + " (short)");
      ticketShort = 0;
      EinstiegShort = 0;
   }
      
   //Antwort des NN verarbeiten
   processNNresponse();
      
   //nur bei einem neuen Bar 
   if(lastBars == Bars) {
      return(0);
   }
      
   lastBars = Bars;
   TrainingCounter++;
   
   handle = FileOpen(Symbol() + Period() + ".csv", FILE_READ);
   if(handle != -1) {
      //die Datei ist noch vorhanden. Sie wird erst nach Abschluss des Lernvorgangs gelscht
      Print("(training)");
      FileClose(handle);
      return(0);
   } else if(training) {
      //Lernmodus beendet
      training = false;
      TrainingCounter = 0;
      Print("Training completed");
   }
         
   //Tickdaten ausgeben
   OpenAtRequest = Open[0];
   handle = FileOpen(Symbol() + Period() + ".i", FILE_CSV|FILE_WRITE);
   writeLine(0, handle, false);
   FileClose(handle);
   Print("Request sent at Open = " + DoubleToStr(Bid, Digits));
   
   //letzten Tick speichern
   handle = FileOpen(Symbol() + Period() + ".prev", FILE_CSV|FILE_WRITE);
   FileWrite(handle, Point, spread, stoplevel);//1. Zeile: Point, Spread, StopLevel
   writeLine(1, handle, true);
   FileClose(handle);
   
   //kurz abwarten, ggf. Antwort verarbeiten
   Sleep(500);
   processNNresponse();
   
//----
   return(0);
  }
  
  
  //Antwort des NN verarbeiten
  void processNNresponse() {
   int err;
  
   //Liegt eine Antwort des NN vor?
   int handle = FileOpen(Symbol() + Period() + ".o", FILE_READ|FILE_CSV);
   if(handle != -1 && !training) {
      //Antwort liegt vor, Lernmodus nicht aktiv - es kann gehandelt werden.
      
      double responseLong = FileReadNumber(handle);
      double stoplossLong = FileReadNumber(handle);
      double responseShort = FileReadNumber(handle);
      double stoplossShort = FileReadNumber(handle);
      
      string desc = "";
      
      if(responseLong == 0) {
         desc = "long: open/hold";
      }
      if(responseLong == 1) {
         desc = desc + "long: off market";
      }
      if(responseShort == 1) {
         desc = desc + "; short: off market";
      }
      if(responseShort == 2) {
         desc = desc + "; short: open/hold";
      }
      
      if(OpenAtRequest != Bid){
         desc = desc + " (slippage = " + DoubleToStr((OpenAtRequest - Bid)/Point,0) + ")";
      }
       
      Print("Response: " + desc);
            
      //trading
      if(IsTradeAllowed()) {
      //***** long *****
         if(responseLong == 0 && ticketLong == 0 && TrainingCounter <= TrainingIntervalBars) {
            //open long, falls keine Lenrphase ansteht                 
            RefreshRates();
            ticketLong = OrderSend(Symbol(), OP_BUY, lots, Ask, 1000, Bid - stoplossLong * Point, 0, "", 0, 0, Green);

            if(!OrderSelect(ticketLong, SELECT_BY_TICKET)) {
               err = GetLastError();
               Print("Ticket #" + ticketLong + " not found (" + err + ")");
               ticketLong = 0;
            } else {
               EinstiegLong = OrderOpenPrice();
               if(EinstiegLong != OpenAtRequest + spread * Point) {
                  Print("Ticket #" + ticketLong + " opened with slippage = " + DoubleToStr((OpenAtRequest + spread * Point - EinstiegLong)/Point, 0));
               }
            }                        
         }
         if(responseLong == 1 && ticketLong != 0) {
            //off market
            RefreshRates();
            if(!OrderClose(ticketLong, lots, Bid, 1000, Red)){
               err = GetLastError();
               Print("Error " + err + " closing ticket #" + ticketLong + " (long)");
               ticketLong = 0;
            } else {
               ticketLong = 0;
               EinstiegLong = 0;
            }
         }
         
         //***** short *****
         if(responseShort == 1 && ticketShort != 0) {
            //off market
            RefreshRates();
            if(!OrderClose(ticketShort, lots, Ask, 1000, Red)){
               err = GetLastError();
               Print("Error " + err + " closing ticket #" + ticketShort + " (short)");
               ticketShort = 0;
            } else {
               ticketShort = 0;
               EinstiegShort = 0;
            }
         }
            
         if(responseShort == 2) {
            //open/hold short
                 
            if(ticketShort == 0 && TrainingCounter <= TrainingIntervalBars) {
               //short ffnen, falls keine Lernphase ansteht
               RefreshRates();
               ticketShort = OrderSend(Symbol(), OP_SELL, lots, Bid, 1000, Ask + stoplossShort * Point, 0, "", 0, 0, Green);

               if(!OrderSelect(ticketShort, SELECT_BY_TICKET)) {
                  err = GetLastError();
                  Print("Ticket #" + ticketShort + " not found (" + err + ")");
                  ticketShort = 0;
               } else {
                  EinstiegShort = OrderOpenPrice();
                  if(OpenAtRequest != EinstiegShort) {
                     Print ("Ticket #" + ticketShort + " opened with slippage = " + DoubleToStr((OpenAtRequest - EinstiegShort)/Point, 0));
                  }
               }            
            }
         }
      } else if(ticketLong != 0 || ticketShort != 0 || responseLong != 1 || responseShort != 1) {
         Print("Trading not allowed");
      }
   }
   
   if(handle != -1) {
      //Steuersatz lschen
      FileClose(handle);
      FileDelete(Symbol() + Period() + ".o");
   }

  }
  
  //eine Zeile mit Daten ausgeben
  void writeLine (int i, int handle, bool learning) {
      
      string c[20];
      
      c[0] = iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+1) - iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+2);
      c[1] = iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+2) - iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+3);
      c[2] = iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+3) - iMA(NULL,0,13,0,MODE_EMA,PRICE_TYPICAL,i+4);
      c[3] = iRSI(NULL,0,13,PRICE_TYPICAL,i+1);
      c[4] = iRSI(NULL,0,13,PRICE_TYPICAL,i+2);
      c[5] = iWPR(NULL,0,13,i+1);
      c[6] = iWPR(NULL,0,13,i+2);
      c[7] = iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_MAIN,i+1);
      c[8] = iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_SIGNAL,i+1) - iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_MAIN,i+1);
      c[9] = iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_MAIN,i+2);
      c[10] = iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_SIGNAL,i+2) - iMACD(NULL,0,12,26,9,PRICE_TYPICAL,MODE_MAIN,i+2);
      c[11] = iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_MAIN,i+1);
      c[12] = iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_SIGNAL,i+1) - iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_MAIN,i+1);
      c[13] = iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_MAIN,i+2);
      c[14] = iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_SIGNAL,i+2) - iStochastic(NULL,0,5,3,3,MODE_EMA,0,MODE_MAIN,i+2);     
          
      if(learning) {
         c[15] = Open[i]; //aktueller Wert. Dieser wird fr die Berechnung des Ergebnisses im Lernmodus verwendet.
         
         //Inputs zhlen bis hierhin!        
         
         c[16] = High[i];
         c[17] = Low[i];
         c[18] = TimeToStr(Time[i]);
      }
      
      string Ausgabe = "";
      int len = ArraySize(c);
      
      if(!learning) {
         len = ArraySize(c) - 4;
      } 
      
      for(int x = 0; x < len; x++) {
         if(StringLen(Ausgabe) > 0) {
            Ausgabe = Ausgabe + ";";
         }
         Ausgabe = Ausgabe + c[x];
      }
      FileWrite(handle, Ausgabe);
      
      if(!learning){
         double loopbackLong = 0;
         double loopbackShort = 0;
         
         //Bisheriges Ergebnis mitgeben
         if(ticketLong != 0) {
            //Einstieg per Ask, Ausstieg per Bid
            loopbackLong = (Open[i] - EinstiegLong) / Point;
         }
         if(ticketShort != 0) {
            //Einstieg per Bid, Ausstieg per Ask
            loopbackShort = (EinstiegShort - (Open[i] + spread * Point)) / Point;
         }
         
         FileWrite(handle, loopbackLong + ";" + loopbackShort);
      }      
  }
//+------------------------------------------------------------------+